unit UMainForm;

{$mode objfpc}
{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, Grids, Menus,
  UEELSBoson, UDiagramm, UFileIO;

type

  { TMainForm }

  TMainForm = class(TForm)
    EELSLabel1: TLabel;
    MainMenu1: TMainMenu;
    FileMenuItem: TMenuItem;
    EELSImportMenuItem: TMenuItem;
    EELSExportMenuItem: TMenuItem;
    BosonImportMenuItem: TMenuItem;
    BosonExportMenuItem: TMenuItem;

    CalculateButton: TButton;
// EELS part 1
    EELSLabel: TLabel;
    ImpactEnergyLabel: TLabel;
    ImpactEnergyEdit:  TEdit;
    IncidenceAngleLabel: TLabel;
    IncidenceAngleEdit:  TEdit;
    AngularAperturePhiALabel: TLabel;
    AngularAperturePhiAEdit:  TEdit;
    AngularAperturePhiBLabel: TLabel;
    AngularAperturePhiBEdit:  TEdit;
    EELSintervalLabel:  TLabel;
    IntervalWMinEdit:   TEdit;
    IntervalWMaxEdit:   TEdit;
    IntervalWDeltaEdit: TEdit;
    Comment1Label: TLabel;
    Comment1Edit:  TEdit;
    Comment2Label: TLabel;
    Comment2Edit:  TEdit;
	
    AddLayerButton: TButton;
    RemoveLayerButton: TButton;
	
// Boson part
    BosonLabel: TLabel;
    TargetTemperatureLabel: TLabel;
    TargetTemperatureEdit:  TEdit;
    InstrumentalWidthLabel: TLabel;
    InstrumentalWidthEdit:  TEdit;
    GaussianFractionLabel:  TLabel;
    GaussianFractionEdit:   TEdit;
    AsymmetryLabel:         TLabel;
    AsymmetryEdit:          TEdit;
    EMinLabel:              TLabel;
    EMinEdit:               TEdit;
    EMaxLabel:              TLabel;
    EMaxEdit:               TEdit;

    procedure FormCreate(Sender: TObject);
	
    procedure EELSImportMenuItemClick(Sender: TObject);
    procedure EELSExportMenuItemClick(Sender: TObject);
    procedure BosonImportMenuItemClick(Sender: TObject);
    procedure BosonExportMenuItemClick(Sender: TObject);

    procedure AddLayerButtonClick(Sender: TObject);
    procedure RemoveLayerButtonClick(Sender: TObject);
    procedure AddOscillatorButtonClick(Sender: TObject);
    procedure RemoveOscillatorButtonClick(Sender: TObject);
	
    procedure CalculateButtonClick(Sender: TObject);

    procedure Convert(Sender: TObject);

  private
    LayerGroup: array of record
      NameLabel: TLabel;
      NameEdit: TEdit;
      ThicknessLabel: TLabel;
      ThicknessEdit: TEdit;
      EpsilonLabel: TLabel;
      EpsilonEdit: TEdit;
      OsciLabel: TLabel;
      OsciStringGrid: TStringGrid;
      AddOscillatorButton: TButton;
      RemoveOscillatorButton: TButton;
    end;

  public

  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  NumberOfLayers := 1;
  NumberOfPeriodic := 1;
  setlength(Comment1,   2*72);
  setlength(Oscillator, 3);
  setlength(LayerGroup, NumberOfLayers);
  setlength(LayerName, NumberOfLayers);
  setlength(Thickness, NumberOfLayers);
  setlength(Epsilon, NumberOfLayers);
  setlength(NumberOfOscillators, 3);

  Layergroup[0].NameLabel := TLabel.Create(self);
  Layergroup[0].NameLabel.Parent := self;
  Layergroup[0].NameLabel.Left := 320;
  Layergroup[0].NameLabel.Height := 16;
  Layergroup[0].NameLabel.Top := 62;
  Layergroup[0].NameLabel.Width := 74;
  Layergroup[0].NameLabel.Caption := 'Layer name:';

  Layergroup[0].NameEdit := TEdit.Create(self);
  Layergroup[0].NameEdit.Parent := self;
  Layergroup[0].NameEdit.Left := 400;
  Layergroup[0].NameEdit.Height := 22;
  Layergroup[0].NameEdit.Top := 56;
  Layergroup[0].NameEdit.Width := 104;
  Layergroup[0].NameEdit.Alignment := taRightJustify;
  Layergroup[0].NameEdit.Text := 'BaTiO3';
  Layergroup[0].NameEdit.Tag := 0;

  Layergroup[0].ThicknessLabel := TLabel.Create(self);
  Layergroup[0].ThicknessLabel.Parent := self;
  Layergroup[0].ThicknessLabel.Left := 320;
  Layergroup[0].ThicknessLabel.Height := 16;
  Layergroup[0].ThicknessLabel.Top := 87;
  Layergroup[0].ThicknessLabel.Width := 97;
  Layergroup[0].ThicknessLabel.Caption := 'Thickness (nm):';

  Layergroup[0].ThicknessEdit := TEdit.Create(self);
  Layergroup[0].ThicknessEdit.Parent := self;
  Layergroup[0].ThicknessEdit.Left := 423;
  Layergroup[0].ThicknessEdit.Height := 22;
  Layergroup[0].ThicknessEdit.Top := 81;
  Layergroup[0].ThicknessEdit.Width := 80;
  Layergroup[0].ThicknessEdit.Alignment := taRightJustify;
  Layergroup[0].ThicknessEdit.Text := '70';
  Layergroup[0].ThicknessEdit.Tag := 0;

  Layergroup[0].EpsilonLabel := TLabel.Create(self);
  Layergroup[0].EpsilonLabel.Parent := self;
  Layergroup[0].EpsilonLabel.Left := 320;
  Layergroup[0].EpsilonLabel.Height := 16;
  Layergroup[0].EpsilonLabel.Top := 112;
  Layergroup[0].EpsilonLabel.Width := 47;
  Layergroup[0].EpsilonLabel.Caption := 'Epsilon:';

  Layergroup[0].EpsilonEdit := TEdit.Create(self);
  Layergroup[0].EpsilonEdit.Parent := self;
  Layergroup[0].EpsilonEdit.Left := 423;
  Layergroup[0].EpsilonEdit.Height := 22;
  Layergroup[0].EpsilonEdit.Top := 106;
  Layergroup[0].EpsilonEdit.Width := 80;
  Layergroup[0].EpsilonEdit.Alignment := taRightJustify;
  Layergroup[0].EpsilonEdit.Text := '8.9';
  Layergroup[0].EpsilonEdit.Tag := 0;

  Layergroup[0].OsciLabel := TLabel.Create(self);
  Layergroup[0].OsciLabel.Parent := self;
  Layergroup[0].OsciLabel.Left := 320;
  Layergroup[0].OsciLabel.Height := 16;
  Layergroup[0].OsciLabel.Top := 135;
  Layergroup[0].OsciLabel.Width := 68;
  Layergroup[0].OsciLabel.Caption := 'Oscillators:';

  Layergroup[0].OsciStringGrid := TStringGrid.Create(self);
  Layergroup[0].OsciStringGrid.Parent := self;
  Layergroup[0].OsciStringGrid.Left := 512;
  Layergroup[0].OsciStringGrid.Height := 94;
  Layergroup[0].OsciStringGrid.Top := 56;
  Layergroup[0].OsciStringGrid.Width := 204;
  Layergroup[0].OsciStringGrid.AutoAdvance := aaRightDown;
  Layergroup[0].OsciStringGrid.ColCount := 3;
  Layergroup[0].OsciStringGrid.Constraints.MaxHeight := 94;
  Layergroup[0].OsciStringGrid.Constraints.MinHeight := 94;
  Layergroup[0].OsciStringGrid.FixedCols := 0;
  Layergroup[0].OsciStringGrid.Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine, goRangeSelect, goEditing, goSmoothScroll];
  Layergroup[0].OsciStringGrid.RowCount := 4;
  Layergroup[0].OsciStringGrid.FixedRows := 1;
  Layergroup[0].OsciStringGrid.ScrollBars := ssVertical;
  Layergroup[0].OsciStringGrid.Rows[0].add('frequ.');
  Layergroup[0].OsciStringGrid.Rows[0].add('q');
  Layergroup[0].OsciStringGrid.Rows[0].add('gam/wto');
  Layergroup[0].OsciStringGrid.Rows[1].add('180');
  Layergroup[0].OsciStringGrid.Rows[1].add('0.23');
  Layergroup[0].OsciStringGrid.Rows[1].add('1e-2');
  Layergroup[0].OsciStringGrid.Rows[2].add('295');
  Layergroup[0].OsciStringGrid.Rows[2].add('6.53');
  Layergroup[0].OsciStringGrid.Rows[2].add('0.05');
  Layergroup[0].OsciStringGrid.Rows[3].add('520');
  Layergroup[0].OsciStringGrid.Rows[3].add('5.03');
  Layergroup[0].OsciStringGrid.Rows[3].add('0.28');
  Layergroup[0].OsciStringGrid.Tag := 0;

  Layergroup[0].AddOscillatorButton := TButton.Create(self);
  Layergroup[0].AddOscillatorButton.Parent := self;
  Layergroup[0].AddOscillatorButton.Left := 423;
  Layergroup[0].AddOscillatorButton.Height := 25;
  Layergroup[0].AddOscillatorButton.Top := 128;
  Layergroup[0].AddOscillatorButton.Width := 40;
  Layergroup[0].AddOscillatorButton.Caption := '+';
  Layergroup[0].AddOscillatorButton.Tag := 0;
  Layergroup[0].AddOscillatorButton.OnClick := @AddOscillatorButtonClick;
//  Layergroup[0].AddOscillatorButton.TabOrder := 16;

  Layergroup[0].RemoveOscillatorButton := TButton.Create(self);
  Layergroup[0].RemoveOscillatorButton.Parent := self;
  Layergroup[0].RemoveOscillatorButton.Left := 464;
  Layergroup[0].RemoveOscillatorButton.Height := 25;
  Layergroup[0].RemoveOscillatorButton.Top := 128;
  Layergroup[0].RemoveOscillatorButton.Width := 40;
  Layergroup[0].RemoveOscillatorButton.Caption := '-';
  Layergroup[0].RemoveOscillatorButton.Tag := 0;
  Layergroup[0].RemoveOscillatorButton.OnClick := @RemoveOscillatorButtonClick;
//  Layergroup[0].RemoveOscillatorButton.TabOrder := 22;

end;

procedure TMainForm.EELSImportMenuItemClick(Sender: TObject);
var
  i, j, index: integer;
begin
  read_eelsin(ImpactEnergy, IncidenceAngle, AngularAperturePhiA, AngularAperturePhiB,
    IntervalWMin, IntervalWMax, IntervalWDelta, Comment1, Comment2,
    NumberOfLayers, NumberOfEpsilons, NumberOfPeriodic, LayerName, Thickness,
    Epsilon, NumberOfOscillators, Oscillator);

  ImpactEnergyEdit.Text        := FormatFloat('0.00', ImpactEnergy);
  IncidenceAngleEdit.Text      := FormatFloat('0.00', IncidenceAngle);
  AngularAperturePhiAEdit.Text := FormatFloat('0.00', AngularAperturePhiA);
  AngularAperturePhiBEdit.Text := FormatFloat('0.00', AngularAperturePhiB);
  IntervalWMinEdit.Text        := FormatFloat('0.00', IntervalWMin);
  IntervalWMaxEdit.Text        := FormatFloat('0.00', IntervalWMax);
  IntervalWDeltaEdit.Text      := FormatFloat('0.00', IntervalWDelta);
  Comment1Edit.Text := Comment1;
  Comment2Edit.Text := Comment2;
// EELS part 2
  if length(Layergroup) > NumberOfLayers then
  begin
    for i := NumberOfLayers to length(Layergroup) - 1 do
    begin
       Layergroup[i].NameLabel.Destroy;
       Layergroup[i].NameEdit.Destroy;
       Layergroup[i].ThicknessLabel.Destroy;
       Layergroup[i].ThicknessEdit.Destroy;
       Layergroup[i].EpsilonLabel.Destroy;
       Layergroup[i].EpsilonEdit.Destroy;
       Layergroup[i].OsciLabel.Destroy;
       Layergroup[i].OsciStringGrid.Destroy;
       Layergroup[i].AddOscillatorButton.Destroy;
       Layergroup[i].RemoveOscillatorButton.Destroy;
    end;
    setlength(Layergroup, NumberOfLayers);
  end
  else if length(Layergroup) < NumberOfLayers then
  begin
    setlength(LayerName, NumberOfLayers);
    setlength(Thickness, NumberOfLayers);
    setlength(Epsilon, NumberOfLayers);
    setlength(NumberOfOscillators, NumberOfLayers);
    i := length(Layergroup);
    repeat
      setlength(LayerGroup, i + 1);
      Layergroup[i].NameLabel := TLabel.Create(self);
      Layergroup[i].NameLabel.Parent := self;
      Layergroup[i].NameLabel.Left := 320;
      Layergroup[i].NameLabel.Height := 16;
      Layergroup[i].NameLabel.Top := 62 + (i) * 100;
      Layergroup[i].NameLabel.Width := 74;
      Layergroup[i].NameLabel.Caption := 'Layer name:';

      Layergroup[i].NameEdit := TEdit.Create(self);
      Layergroup[i].NameEdit.Parent := self;
      Layergroup[i].NameEdit.Left := 400;
      Layergroup[i].NameEdit.Height := 22;
      Layergroup[i].NameEdit.Top := 56 + (i) * 100;
      Layergroup[i].NameEdit.Width := 104;
      Layergroup[i].NameEdit.Alignment := taRightJustify;
      Layergroup[i].NameEdit.Tag := i;

      Layergroup[i].ThicknessLabel := TLabel.Create(self);
      Layergroup[i].ThicknessLabel.Parent := self;
      Layergroup[i].ThicknessLabel.Left := 320;
      Layergroup[i].ThicknessLabel.Height := 16;
      Layergroup[i].ThicknessLabel.Top := 87 + (i) * 100;
      Layergroup[i].ThicknessLabel.Width := 97;
      Layergroup[i].ThicknessLabel.Caption := 'Thickness (nm):';

      Layergroup[i].ThicknessEdit := TEdit.Create(self);
      Layergroup[i].ThicknessEdit.Parent := self;
      Layergroup[i].ThicknessEdit.Left := 423;
      Layergroup[i].ThicknessEdit.Height := 22;
      Layergroup[i].ThicknessEdit.Top := 81 + (i) * 100;
      Layergroup[i].ThicknessEdit.Width := 80;
      Layergroup[i].ThicknessEdit.Alignment := taRightJustify;
      Layergroup[i].ThicknessEdit.Tag := i;

      Layergroup[i].EpsilonLabel := TLabel.Create(self);
      Layergroup[i].EpsilonLabel.Parent := self;
      Layergroup[i].EpsilonLabel.Left := 320;
      Layergroup[i].EpsilonLabel.Height := 16;
      Layergroup[i].EpsilonLabel.Top := 112 + (i) * 100;
      Layergroup[i].EpsilonLabel.Width := 47;
      Layergroup[i].EpsilonLabel.Caption := 'Epsilon:';

      Layergroup[i].EpsilonEdit := TEdit.Create(self);
      Layergroup[i].EpsilonEdit.Parent := self;
      Layergroup[i].EpsilonEdit.Left := 423;
      Layergroup[i].EpsilonEdit.Height := 22;
      Layergroup[i].EpsilonEdit.Top := 106 + (i) * 100;
      Layergroup[i].EpsilonEdit.Width := 80;
      Layergroup[i].EpsilonEdit.Alignment := taRightJustify;
      Layergroup[i].EpsilonLabel.Tag := i;

      Layergroup[i].OsciLabel := TLabel.Create(self);
      Layergroup[i].OsciLabel.Parent := self;
      Layergroup[i].OsciLabel.Left := 320;
      Layergroup[i].OsciLabel.Height := 16;
      Layergroup[i].OsciLabel.Top := 135 + (i) * 100;
      Layergroup[i].OsciLabel.Width := 68;
      Layergroup[i].OsciLabel.Caption := 'Oscillators:';

      Layergroup[i].OsciStringGrid := TStringGrid.Create(self);
      Layergroup[i].OsciStringGrid.Parent := self;
      Layergroup[i].OsciStringGrid.Left := 512;
      Layergroup[i].OsciStringGrid.Height := 94;
      Layergroup[i].OsciStringGrid.Top := 56 + (i) * 100;
      Layergroup[i].OsciStringGrid.Width := 204;
      Layergroup[i].OsciStringGrid.AutoAdvance := aaRightDown;
      Layergroup[i].OsciStringGrid.ColCount := 3;
      Layergroup[i].OsciStringGrid.Constraints.MaxHeight := 94;
      Layergroup[i].OsciStringGrid.Constraints.MinHeight := 94;
      Layergroup[i].OsciStringGrid.FixedCols := 0;
      Layergroup[i].OsciStringGrid.Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine, goRangeSelect, goEditing, goSmoothScroll];
      Layergroup[i].OsciStringGrid.RowCount := 2;
      Layergroup[i].OsciStringGrid.RowCount := NumberOfOscillators[i] + 1;
      Layergroup[i].OsciStringGrid.FixedRows := 1;
      Layergroup[i].OsciStringGrid.ScrollBars := ssVertical;
      Layergroup[i].OsciStringGrid.Rows[0].add('frequ.');
      Layergroup[i].OsciStringGrid.Rows[0].add('q');
      Layergroup[i].OsciStringGrid.Rows[0].add('gam/wto');
      Layergroup[i].OsciStringGrid.Tag := i;

      Layergroup[i].AddOscillatorButton := TButton.Create(self);
      Layergroup[i].AddOscillatorButton.Parent := self;
      Layergroup[i].AddOscillatorButton.Left := 423;
      Layergroup[i].AddOscillatorButton.Height := 25;
      Layergroup[i].AddOscillatorButton.Top := 128 + (i) * 100;
      Layergroup[i].AddOscillatorButton.Width := 40;
      Layergroup[i].AddOscillatorButton.Caption := '+';
      Layergroup[i].AddOscillatorButton.Tag := i;
      Layergroup[i].AddOscillatorButton.OnClick := @AddOscillatorButtonClick;
    //  Layergroup[i].AddOscillatorButton.TabOrder := 16;

      Layergroup[i].RemoveOscillatorButton := TButton.Create(self);
      Layergroup[i].RemoveOscillatorButton.Parent := self;
      Layergroup[i].RemoveOscillatorButton.Left := 464;
      Layergroup[i].RemoveOscillatorButton.Height := 25;
      Layergroup[i].RemoveOscillatorButton.Top := 128 + (i) * 100;
      Layergroup[i].RemoveOscillatorButton.Width := 40;
      Layergroup[i].RemoveOscillatorButton.Caption := '-';
      Layergroup[i].RemoveOscillatorButton.Tag := i;
      Layergroup[i].RemoveOscillatorButton.OnClick := @RemoveOscillatorButtonClick;
    //  Layergroup[i].RemoveOscillatorButton.TabOrder := 22;

      inc(i);
    until (length(Layergroup) = NumberOfLayers);
  end;
  for i := 0 to NumberOfLayers - 1 do
  begin
    Layergroup[i].NameEdit.Text := LayerName[i];
    Layergroup[i].ThicknessEdit.Text := FormatFloat('0.00',Thickness[i]);
    Layergroup[i].EpsilonEdit.Text    := FormatFloat('0.00',Epsilon[i]);
    Layergroup[i].OsciStringGrid.RowCount := NumberOfOscillators[i] + 1;
  end;
  index := 0;
  for i := 0 to NumberOfLayers -1 do
    for j := 1 to NumberOfOscillators[i] do
    begin
       Layergroup[i].OsciStringGrid.Cells[0,j] := FormatFloat('0.000', Oscillator[index,1]);
       Layergroup[i].OsciStringGrid.Cells[1,j] := FormatFloat('0.000', Oscillator[index,2]);
       Layergroup[i].OsciStringGrid.Cells[2,j] := FormatFloat('0.000', Oscillator[index,3]);
       inc(index);
    end;
end;

procedure TMainForm.EELSExportMenuItemClick(Sender: TObject);
begin
  Convert(self);
  write_eelsin(ImpactEnergy, IncidenceAngle, AngularAperturePhiA, AngularAperturePhiB,
    IntervalWMin, IntervalWMax, IntervalWDelta, Comment1, Comment2,
    NumberOfLayers, NumberOfEpsilons, NumberOfPeriodic, LayerName, Thickness,
    Epsilon, NumberOfOscillators, Oscillator);
end;

procedure TMainForm.BosonImportMenuItemClick(Sender: TObject);
begin
  read_bosin(TargetTemperature, InstrumentalWidth,
             GaussianFraction, Asymmetry, EMin, EMax);
end;

procedure TMainForm.BosonExportMenuItemClick(Sender: TObject);
begin
  Convert(self);
  write_bosin(TargetTemperature, InstrumentalWidth,
              GaussianFraction, Asymmetry, EMin, EMax);
end;

procedure TMainForm.AddLayerButtonClick(Sender: TObject);
begin
  inc(NumberOfLayers);
  inc(NumberOfPeriodic);
  setlength(LayerGroup, NumberOfLayers);
  setlength(LayerName, NumberOfLayers);
  setlength(Thickness, NumberOfLayers);
  setlength(Epsilon, NumberOfLayers);
  setlength(NumberOfOscillators, NumberOfLayers);

  Layergroup[NumberOfLayers - 1].NameLabel := TLabel.Create(self);
  Layergroup[NumberOfLayers - 1].NameLabel.Parent := self;
  Layergroup[NumberOfLayers - 1].NameLabel.Left := 320;
  Layergroup[NumberOfLayers - 1].NameLabel.Height := 16;
  Layergroup[NumberOfLayers - 1].NameLabel.Top := 62 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].NameLabel.Width := 74;
  Layergroup[NumberOfLayers - 1].NameLabel.Caption := 'Layer name:';

  Layergroup[NumberOfLayers - 1].NameEdit := TEdit.Create(self);
  Layergroup[NumberOfLayers - 1].NameEdit.Parent := self;
  Layergroup[NumberOfLayers - 1].NameEdit.Left := 400;
  Layergroup[NumberOfLayers - 1].NameEdit.Height := 22;
  Layergroup[NumberOfLayers - 1].NameEdit.Top := 56 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].NameEdit.Width := 104;
  Layergroup[NumberOfLayers - 1].NameEdit.Alignment := taRightJustify;
  Layergroup[NumberOfLayers - 1].NameEdit.Tag := NumberOfLayers - 1;

  Layergroup[NumberOfLayers - 1].ThicknessLabel := TLabel.Create(self);
  Layergroup[NumberOfLayers - 1].ThicknessLabel.Parent := self;
  Layergroup[NumberOfLayers - 1].ThicknessLabel.Left := 320;
  Layergroup[NumberOfLayers - 1].ThicknessLabel.Height := 16;
  Layergroup[NumberOfLayers - 1].ThicknessLabel.Top := 87 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].ThicknessLabel.Width := 97;
  Layergroup[NumberOfLayers - 1].ThicknessLabel.Caption := 'Thickness (nm):';

  Layergroup[NumberOfLayers - 1].ThicknessEdit := TEdit.Create(self);
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Parent := self;
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Left := 423;
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Height := 22;
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Top := 81 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Width := 80;
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Alignment := taRightJustify;
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Text := '0';
  Layergroup[NumberOfLayers - 1].ThicknessEdit.Tag := NumberOfLayers - 1;

  Layergroup[NumberOfLayers - 1].EpsilonLabel := TLabel.Create(self);
  Layergroup[NumberOfLayers - 1].EpsilonLabel.Parent := self;
  Layergroup[NumberOfLayers - 1].EpsilonLabel.Left := 320;
  Layergroup[NumberOfLayers - 1].EpsilonLabel.Height := 16;
  Layergroup[NumberOfLayers - 1].EpsilonLabel.Top := 112 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].EpsilonLabel.Width := 47;
  Layergroup[NumberOfLayers - 1].EpsilonLabel.Caption := 'Epsilon:';

  Layergroup[NumberOfLayers - 1].EpsilonEdit := TEdit.Create(self);
  Layergroup[NumberOfLayers - 1].EpsilonEdit.Parent := self;
  Layergroup[NumberOfLayers - 1].EpsilonEdit.Left := 423;
  Layergroup[NumberOfLayers - 1].EpsilonEdit.Height := 22;
  Layergroup[NumberOfLayers - 1].EpsilonEdit.Top := 106 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].EpsilonEdit.Width := 80;
  Layergroup[NumberOfLayers - 1].EpsilonEdit.Alignment := taRightJustify;
  Layergroup[NumberOfLayers - 1].EpsilonEdit.Text := '0';
  Layergroup[NumberOfLayers - 1].EpsilonLabel.Tag := NumberOfLayers - 1;

  Layergroup[NumberOfLayers - 1].OsciLabel := TLabel.Create(self);
  Layergroup[NumberOfLayers - 1].OsciLabel.Parent := self;
  Layergroup[NumberOfLayers - 1].OsciLabel.Left := 320;
  Layergroup[NumberOfLayers - 1].OsciLabel.Height := 16;
  Layergroup[NumberOfLayers - 1].OsciLabel.Top := 135 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].OsciLabel.Width := 68;
  Layergroup[NumberOfLayers - 1].OsciLabel.Caption := 'Oscillators:';

  Layergroup[NumberOfLayers - 1].OsciStringGrid := TStringGrid.Create(self);
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Parent := self;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Left := 512;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Height := 94;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Top := 56 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Width := 204;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.AutoAdvance := aaRightDown;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.ColCount := 3;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Constraints.MaxHeight := 94;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Constraints.MinHeight := 94;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.FixedCols := 0;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine, goRangeSelect, goEditing, goSmoothScroll];
  Layergroup[NumberOfLayers - 1].OsciStringGrid.RowCount := 2;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.FixedRows := 1;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.ScrollBars := ssVertical;
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Rows[0].add('frequ.');
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Rows[0].add('q');
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Rows[0].add('gam/wto');
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Rows[1].add('0');
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Rows[1].add('0');
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Rows[1].add('0');
  Layergroup[NumberOfLayers - 1].OsciStringGrid.Tag := NumberOfLayers - 1;

  Layergroup[NumberOfLayers - 1].AddOscillatorButton := TButton.Create(self);
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.Parent := self;
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.Left := 423;
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.Height := 25;
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.Top := 128 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.Width := 40;
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.Caption := '+';
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.Tag := NumberOfLayers - 1;
  Layergroup[NumberOfLayers - 1].AddOscillatorButton.OnClick := @AddOscillatorButtonClick;
//  Layergroup[NumberOfLayers - 1].AddOscillatorButton.TabOrder := 16;

  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton := TButton.Create(self);
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.Parent := self;
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.Left := 464;
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.Height := 25;
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.Top := 128 + (NumberOfLayers - 1) * 100;
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.Width := 40;
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.Caption := '-';
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.Tag := NumberOfLayers - 1;
  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.OnClick := @RemoveOscillatorButtonClick;
//  Layergroup[NumberOfLayers - 1].RemoveOscillatorButton.TabOrder := 22;

end;

procedure TMainForm.RemoveLayerButtonClick(Sender: TObject);
begin
  if NumberOfLayers > 1 then
  begin
    with Layergroup[NumberOfLayers - 1] do
    begin
      NameLabel.Destroy;
      NameEdit.Destroy;
      ThicknessLabel.Destroy;
      ThicknessEdit.Destroy;
      EpsilonLabel.Destroy;
      EpsilonEdit.Destroy;
      OsciLabel.Destroy;
      OsciStringGrid.Destroy;
      AddOscillatorButton.Destroy;
      RemoveOscillatorButton.Destroy;
    end;
    dec(NumberOfLayers);
    dec(NumberOfPeriodic);
    setlength(LayerGroup, NumberOfLayers);
    setlength(LayerName, NumberOfLayers);
    setlength(Thickness, NumberOfLayers);
    setlength(Epsilon, NumberOfLayers);
    setlength(NumberOfOscillators, NumberOfLayers);
  end;
end;
procedure TMainForm.AddOscillatorButtonClick(Sender: TObject);
var
  index: integer;
begin
  index := TButton(Sender).Tag;
  with Layergroup[index] do
  begin
    OsciStringGrid.InsertRowWithValues(OsciStringGrid.RowCount, ['0','0','0']);
    OsciStringGrid.Height := OsciStringGrid.Height + 23;
    inc(NumberOfOscillators[index]);
  end;
end;
procedure TMainForm.RemoveOscillatorButtonClick(Sender: TObject);
var
  index: integer;
begin
  index := TButton(Sender).Tag;
  with Layergroup[index] do
    if OsciStringGrid.RowCount > 2 then // keep at least one oscillator
    begin
      OsciStringGrid.DeleteRow(OsciStringGrid.RowCount - 1);
      OsciStringGrid.Height := OsciStringGrid.Height - 23;
      dec(NumberOfOscillators[index]);
    end;
end;

procedure TMainForm.CalculateButtonClick(Sender: TObject);
var
  i, j, index: integer;
begin
  Convert(self);
  doeels(ImpactEnergy, IncidenceAngle, AngularAperturePhiA, AngularAperturePhiB,
         IntervalWMin, IntervalWMax, IntervalWDelta, Comment1, Comment1_length,
         NumberOfLayers, NumberOfEpsilons, NumberOfPeriodic, LayerName, LayerName_length,
         Thickness, Epsilon, NumberOfOscillators, Oscillator, Oscillator_length,
         Control, Mode, Wn_Array, F_Array, IntervalNW);

  doboson(TargetTemperature, InstrumentalWidth, GaussianFraction, Asymmetry, EMin, EMax,
          IntervalWMin, IntervalWMax, IntervalNW, F_Array, xout, yout, nout);

  DiagrammForm.Visible := true;
end;

procedure TMainForm.Convert(Sender: TObject);
var
  i, j, index: integer;
begin
  NumberOfEpsilons := NumberOfLayers;
  if NumberOfPeriodic = -1 then
  begin
    NumberOfEpsilons := NumberOfLayers + 1;
    NumberOfPeriodic := 1;
  end;
// EELS part 1
  TryStrToFloat(ImpactEnergyEdit.Text, ImpactEnergy);
  TryStrToFloat(IncidenceAngleEdit.Text, IncidenceAngle);
  TryStrToFloat(AngularAperturePhiAEdit.Text, AngularAperturePhiA);
  TryStrToFloat(AngularAperturePhiBEdit.Text, AngularAperturePhiB);
  TryStrToFloat(IntervalWMinEdit.Text, IntervalWMin);
  TryStrToFloat(IntervalWMaxEdit.Text, IntervalWMax);
  TryStrToFloat(IntervalWDeltaEdit.Text, IntervalWDelta);
  Comment1 := Comment1Edit.Text;
  Comment2 := Comment2Edit.Text;
  Comment1_length := 2*72;
// EELS part 2
  LayerName_length := 10;
  Oscillator_length := 0;
  for i := 0 to NumberOfLayers -1 do
  begin
    LayerName[i] := Layergroup[i].NameEdit.Text;
    TryStrToFloat(Layergroup[i].ThicknessEdit.Text, Thickness[i]);
    TryStrToFloat(Layergroup[i].EpsilonEdit.Text, Epsilon[i]);
    NumberOfOscillators[i] := Layergroup[i].OsciStringGrid.RowCount - 1;
    Oscillator_length := Oscillator_length + NumberOfOscillators[i];
  end;
  setlength(Oscillator, Oscillator_length);
  index := 0;
  for i := 0 to NumberOfLayers -1 do
    for j := 1 to NumberOfOscillators[i] do
    begin
       TryStrToFloat(Layergroup[i].OsciStringGrid.Cells[0,j],Oscillator[index,1]);
       TryStrToFloat(Layergroup[i].OsciStringGrid.Cells[1,j],Oscillator[index,2]);
       TryStrToFloat(Layergroup[i].OsciStringGrid.Cells[2,j],Oscillator[index,3]);
       inc(index);
    end;

  IntervalNW := 1 + round((IntervalWMax - IntervalWMin) / IntervalWDelta);
  setlength(Wn_Array, IntervalNW);
  setlength(F_Array, IntervalNW);
  Control := ' ';
  Mode := ' ';

// Boson part
  TryStrToFloat(TargetTemperatureEdit.Text, TargetTemperature);
  TryStrToFloat(InstrumentalWidthEdit.Text, InstrumentalWidth);
  TryStrToFloat(GaussianFractionEdit.Text, GaussianFraction);
  TryStrToFloat(AsymmetryEdit.Text, Asymmetry);
  TryStrToFloat(EMinEdit.Text, EMin);
  TryStrToFloat(EMaxEdit.Text, EMax);
  nout := 3000;
  setlength(xout, nout);
  setlength(yout, nout);
end;
end.

