program eels

! ******************************************************************
! *                                                                *
! * compute the classical eels spectrum of an arbitrary plane-     *
! * statified medium made from isotropic materials in specular     *
! * geometry using the dielectric theory of eels.                  *
! *                                                                *
! ******************************************************************

  implicit none

  double precision :: e0, theta, phia, phib, wmin, wmax, dw
  integer :: i, j, jos, k, l, layers, neps, nper, nw
  logical :: user
  character (len = 72) :: comment(2)
  character (len = 10) :: contrl, mode

  double precision, allocatable :: thick(:), epsinf(:), osc(:, :)
  integer, allocatable :: nos(:)
  character (len = 10), allocatable :: name(:)
  double precision, allocatable :: wn_array(:), f(:)

  integer :: old_size_1, old_size_2
  double precision, allocatable :: tmp_osc(:, :)
  integer :: ioStatus

!  integer, parameter :: name_length = 10
! *** read spectrometer parameters

  call change_working_dir()
  open(unit = 11, file = 'eelsin')
! impact energy (ev)
  read(11, *) e0
! incidence angle (%)
  read(11, *) theta
! angular apertures of the elliptic detector (%)
  read(11, *) phia
  read(11, *) phib
! energy-loss interval and step size (cm**-1)
  read(11, *) wmin
  read(11, *) wmax
  read(11, *) dw
! comment lines
  read(11, '(a72)') (comment(k), k = 1, 2)

  write(*,*) 'program eels (September 2020)'
  write(*,'(a, f6.2, a, f5.1, a, f5.2, a, f5.2, a)') &
    ' e0 = ', e0, ' eV, theta = ', theta, '°, phia = ', &
    phia, '°, phib = ', phib, '°'
  write(*,'(a, g11.4, a, g11.4, a, g11.4, a)') &
    ' energy losses from', wmin, ' to', wmax, ', step = ', dw, ' cm**-1'
  write(*,*) comment(1)
  write(*,*) comment(2)

  if (phia <= 0.0d0 .or. phib <= 0.0d0) then
    stop '*** wrong input ***'
  endif
  if (e0 <= 0.0d0 .or. theta + phia >= 90.0d0) then
    stop '*** bad input ***'
  endif

! *** read target specifications

  read(11, *) layers, nper, mode
  user = layers == 0
  if (.not. user) then
    neps = layers
    if (nper == -1) then
      neps = layers + 1
      nper = 1
      write(*,*) 'the substrate is a anisotropic uniaxial material'
    endif
    if (layers < 0 .or. nper < 1 .or. nper > layers) then
      stop '*** invalid target specifications ***'
    endif
    write(6, 101) layers, nper
    jos = 0
    allocate (name(neps), thick(neps), epsinf(neps), nos(neps))
    do l = 1, neps
      if (l <= layers) then
        read(11, 102) name(l), thick(l)
      endif
      read(11, *) epsinf(l), nos(l)
      write(6, 103)
      if (nos(l) <= 0) then
        if (l <= layers) write(6, 104) l, name(l), thick(l), epsinf(l)
        if (l  > layers) write(6, 105) epsinf(l)
      else
        do j = 1, nos(l)
          if (.not. allocated(osc)) then
            allocate(osc(3, nos(l)))
          else if (j == 1) then
 !           call extend3(osc, nos(j))
            old_size_1 = size(osc, 1)
            old_size_2 = size(osc, 2)
            allocate(tmp_osc(old_size_1, old_size_2 + nos(l)))
            tmp_osc(1:old_size_1, 1:old_size_2) = osc
            deallocate(osc)
            call move_alloc(tmp_osc, osc)
          endif
          jos = jos + 1
          read(11, *) (osc(k, jos), k = 1, 3)
          if ((j == nos(l) / 2 + 1) .and. (nos(l) > 1)) then
            write(6, 106)
            write(6, 107)
          endif
          if (j == 1) then
            if (l <= layers) then
              write(6, 104) l, name(l), thick(l), epsinf(l), (osc(i, jos), i = 1, 3)
            else
              write(6, 105) epsinf(l), (osc(i, jos), i = 1, 3)
            endif
          else
            write(6, 108) (osc(i, jos), i = 1, 3)
          endif
        enddo
      endif
    enddo
    write(*,*)
    read(11, 102, IOSTAT = ioStatus) contrl
    if (ioStatus /= 0) then
      contrl = ''
    endif
  endif

  close (unit = 11)

  nw = 1 + int((wmax - wmin) / dw)
  allocate (wn_array(nw), f(nw))

  call doeels(e0, theta, phia, phib, wmin, wmax, dw, comment, size(comment),   &
              layers, neps, nper, name, size(name), thick, epsinf, nos, osc, size(osc, 2),&
              contrl, mode, wn_array, f, size(wn_array))

  open (unit = 12, file = 'eelsou')
  write (12, 207) e0, theta, phia, phib, comment(1)
  do i = 1, nw
    write (12, 211) wn_array(i), f(i)
  enddo
  close (unit = 12)

  stop

101 format(i3, ' layer(s), nper = ', i2//'   l', 2x, 'material', 7x,  &
      'thickness', 5x, 'epsinf', 4x, 'wto , wp', 5x, 'q', 7x, 'gam/wto')
102 format(a10, d15.5)
103 format(1x, 72('-'))
104 format(1x, i3, 2x, a10, g15.3, f10.4, f12.4, f10.4, f9.4)
105 format(31x, f10.4, f12.4, f10.4, f9.4)
106 format(45x, 'wlo , wp', 5x, 'q', 7x, 'gam/wlo')
107 format(45x, 28('-'))
108 format(41x, f12.4, f10.4, f9.4)
207 format('e0 =', f6.2, ' theta =', f5.1, ' phia =', f5.2, ' phib =', f5.2 / a72)
211 format(2e15.7)
end program eels

double precision function qrat(x, alpha, beta, c1, c2)

  implicit none

  double precision, intent(in) :: x, alpha, beta, c1, c2

  qrat = (1.0d0 + x * (beta + c1 * x)) / ((1.0d0 + x * (beta + c2 * x)) * (1.0d0 + alpha * x)**2)

  return
end function qrat

double precision function usurlo(dq, wn)

! ******************************************************************
! *                                                                *
! * user-supplied dielectric surface loss function aimag(g(dq, wn)) *
! * input arguments :                                              *
! *    dq : modulus of the two-dimensional surface wave vector     *
! *         (angstroem**-1)                                        *
! *    wn : frequency (cm**-1)                                     *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: dq
  double precision, intent(in) :: wn

  usurlo = 1.0d0
  return
end function usurlo
  
double precision function surlos(dk, eps, thick, layers, nper)

! ******************************************************************
! *                                                                *
! * eels surface loss function for an arbitrary multilayered target*
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: dk
  double complex, intent(in) :: eps(:)
  double precision, intent(in) :: thick(:)
  integer, intent(in) :: layers, nper

  integer :: lmax
  logical :: static, zero, skip
  integer :: lstart, n
  double precision, allocatable :: arg(:)
  double precision :: argmin, argmax, cn, cnm1, epsmac, sn, snm1, tn
  double complex :: a, b, csi, pnm2, pnm1, pn, pp, qnm2, qnm1, qn, qp, z

  common / mulayr / argmin, argmax, epsmac

  zero(z) = (dble(z) == 0.0d0) .and. (dimag(z) == 0.0d0)

!  write (*,*) 'surlos:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  lmax = size(eps)
  allocate (arg(lmax))
  lstart = layers - nper + 1
  static = .true.
  skip = .false.

  do n = 1, layers
    arg(n) = dk * thick(n)
    if (arg(n) > argmax .or. zero(eps(n))) then
      csi  = eps(n)
      skip = .true.
      exit
    endif
    static = .not. (n >= lstart .and. arg(n) > argmin)
  enddo

  if (.not. skip) then
! *** periodic continued fraction, period = nper

    do ! Do loop is only for the option to skip out of it.
      if (nper <= 1) then
        csi = eps(layers)
      else
        if (static) then
          pn = 0.0d0
          qn = 0.0d0
          do n = lstart, layers
            pn = pn + thick(n) * eps(n)
            qn = qn + thick(n) / eps(n)
          enddo
          if (zero(qn)) then
            n = lstart
            if (n <= 1) then
              surlos = 0.0d0
              return
            endif
            n = n - 1
            csi = eps(n) / dtanh(arg(n))
            exit
          endif ! zero(qn)
          csi = cdsqrt(pn / qn)
          if ((dimag(csi) < 0.0d0) .and. (dble(qn) < 0.0d0)) then
            csi = -csi
          endif
        else ! static
          cn = dcosh(arg(lstart))
          sn = dsinh(arg(lstart))
          pnm1 = 1.0d0
          pn = cn
          pp = eps(lstart) * sn
          qnm1 = 0.0d0
          qn = sn / eps(lstart)
          qp = pn
          do  n = lstart + 1, layers
            cnm1 = cn
            snm1 = sn
            cn = dcosh(arg(n))
            sn = dsinh(arg(n))
            a = eps(n) * sn
            pp = cn * pp + a * pn
            qp = cn * qp + a * qn
            b = (eps(n - 1) / eps(n)) * (sn / snm1)
            a = cnm1 * b + cn
            pnm2 = pnm1
            pnm1 = pn
            qnm2 = qnm1
            qnm1 = qn
            pn = a * pnm1 - b * pnm2
            qn = a * qnm1 - b * qnm2
          enddo
          if (zero(qn)) then
            a = qp - pn
            if (zero(a)) then
              n = lstart
              if (n <= 1) then
                surlos = 0.0d0
                return
              endif
              n = n - 1
              csi = eps(n) / dtanh(arg(n))
              exit
            endif
            csi = pp / a
          else
            a = 0.5d0 * (pn - qp) / qn
            b = cdsqrt(a**2 + pp / qn)
            pn = a - pn / qn
            if (cdabs(pn + b) > cdabs(pn - b)) then
              b = -b
            endif
            csi = a + b
          endif ! zero(qn)
        endif ! static
! *** small-dk limit of the periodic tail

      endif ! nper <= 1

      n = lstart

      exit
    enddo
  endif ! .not. skip

! *** backward algorithm

  do
    n = n - 1
    if (n <= 0) then
      a = csi + 1.0d0
      if (zero(a)) then
        surlos = 2 / epsmac
      else
        surlos = dimag(-2 / a)
      endif
      return
    endif ! n <= 0

    if (arg(n) /= 0.0d0) then
      tn = dtanh(arg(n))
      b = eps(n) + csi * tn
      if (zero(b)) then
        surlos = 0.0d0
        return
      endif
      csi = eps(n) * (csi + tn * eps(n)) / b
    endif
  enddo
end function surlos

double precision function phint(phi, a, u)

! ******************************************************************
! *                                                                *
! * evaluate the integral from zero to phi of                      *
! *                                                                *
! *                 u                 2                            *
! *  ( ----------------------------- )  dphi                       *
! *                          2     2                               *
! *    (1 - a * u * cos(phi))  +  u                                *
! *                                                                *
! * for 0 <= phi <= pi , u >= 0 and a >= 0                         *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: phi
  double precision, intent(in) :: a
  double precision, intent(in) :: u

  double precision :: ai, ar, bi, br, c, cpr, d, e, esr, pi, qr, ri, rm, root
  double precision :: rp, rr, s, spr, tm, tp, u2, x, zeta, zetai, zetar, zr

  pi = 3.141592653589793238d0
  c = dcos(phi)
  s = dsin(phi)
  u2 = u**2
  e = a*u
  if (u < 1.0d0 .and. e < 1.0d-02 * (1.0d0 + u2)) then
    zr = 1.0d0 + u2
    esr = e / zr
    phint = u2 / zr**2 * ((( (4.0d0 / 3.0d0) * (2.0d0 + c**2) * s * (5.0d0 - 3 * u2) *  &
            esr + (phi + c * s) * (5.0d0 - u2)) * esr + 4 * s) * esr + phi)
  else
    rm = dsqrt((1.0d0 - e)**2 + u2)
    tm = 0.5d0 * datan2(u, 1.0d0 - e)
    rp = dsqrt((1.0d0 + e)**2 + u2)
    tp = 0.5d0 * datan2(u, 1.0d0 + e)
    root = dsqrt(rm * rp)
    cpr = dcos(tm + tp)
    spr = dsin(tm + tp)
    x = 0.0d0       ! ensure initialization
    if (c >= 0.0d0) then
      x = s / (1.0d0 + c)
    elseif (dabs(s) > 1.0d-07) then
      x = (1.0d0 - c) / s
    endif
    if ((c >= 0.0d0) .or. (dabs(s) > 1.0d-07)) then
      zeta = dsqrt(rm / rp)
      zetar = -zeta * dsin(tm - tp)
      zetai =  zeta * dcos(tm - tp)
      br = 0.5d0 * dlog(((zetar + x)**2 + zetai**2) / ((zetar - x)**2 + zetai**2))
      bi = datan2(zetai, zetar + x) - datan2(zetai, zetar - x)
      rr = -(br * spr - bi * cpr) / root
      ri = -(bi * spr + br * cpr) / root
      d = e * s / ((1.0d0 - e * c)**2 + u2)
      ar = d * (1.0d0 - e * c) - rr + u * ri
      ai = -d * u - ri - u * rr
    else
      rr = -pi / root * cpr
      ri =  pi / root * spr
      ar = -rr + u * ri
      ai = -ri - u * rr
    endif
    qr = (ar * (cpr - spr) * (cpr + spr) + 2 * ai * cpr * spr) / (rm * rp)
    phint = 0.5d0 * (ri / u - qr)
  endif
  return
end function phint

double precision function fint1(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from 0.0 to pi             *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: u
  integer, intent(in) :: layers, nper, eps_size
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)

  logical :: rational, user
  double precision :: acoef, bcoef, ccoef, cospsi, den, dif, dlimf, e, elleps
  double precision :: pi, rom, rop, ru, sinpsi, sum, um, t
  double precision :: tanpsi, wn, u2

  interface
    double precision function usurlo(dq, wn)
      double precision, intent(in) :: dq
      double precision, intent(in) :: wn
    end function usurlo
    double precision function surlos(dk, eps, thick, layers, nper)
      double precision, intent(in) :: dk
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end function surlos
  end interface  

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

  data pi / 3.141592653589793238d0 /

!  write (*,*) 'fint1:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  if (u == 0.0d0) then
    fint1 = 0.0d0
    return
  endif
  e = tanpsi * u
  u2 = u**2
  rom = (1.0d0 - e)**2 + u2
  rop = (1.0d0 + e)**2 + u2
  sum = rop + rom
  rom = dsqrt(rom)
  rop = dsqrt(rop)
  dif = rop - rom
  den = dsqrt((2.0d0 - dif) * (2.0d0 + dif)) * rop * rom
  fint1 = pi * u2 * (4 * sum - dif**2 * (sum - rop * rom)) / den**3
  if (rational) then
    return
  endif
  if (user) then
    fint1 = fint1 * usurlo(ru * u, wn)
  else
    fint1 = fint1 * surlos(ru * u, eps, thick, layers, nper)
    if (dlimf > 0.0d0) then
      t = ru * u * dlimf
      fint1 = fint1 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
    endif
  endif
  return
end function fint1

double precision function fint2(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from 0.0 to phi < pi       *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: u
  integer, intent(in) :: layers, nper, eps_size
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)

  logical :: rational, user
  double precision :: a, arg, b, b2, c, ccoef, cospsi, dlimf, elleps, phi
  double precision :: phint, ru, sinpsi, um, t, tanpsi, wn, x

  interface
    double precision function usurlo(dq, wn)
      double precision, intent(in) :: dq
      double precision, intent(in) :: wn
    end function usurlo
    double precision function surlos(dk, eps, thick, layers, nper)
      double precision, intent(in) :: dk
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end function surlos
  end interface  

  common / param / a, b, ccoef, elleps, cospsi, sinpsi, tanpsi, &
      ru, um, dlimf, wn, user, rational

!  write (*,*) 'fint2:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  if (u == 0.0d0) then
    fint2 = 0.0d0
    return
  endif
  b2 = b**2
  c = (1.0d0 - elleps) * (cospsi * u)**2 + (b - ccoef) * (b + ccoef)
  if (dabs(a * c) > 1.0d-03 * b2) then
    x = (b - dsqrt(b2 - a * c)) / a
  else
    x = a * c / b2
    x = 0.5d0 * c * (1.d0 + 0.25d0 * x * (1.d0 + 0.5d0 * x * (1.d0 + 0.625d0 * x))) / b
  endif
  arg = x / u
  if (dabs(arg) > 1.0d0) then
    arg = dsign(1.0d0, arg)
  endif
  phi = dacos(arg)
  fint2 = phint(phi, tanpsi, u)
  if (rational) then
    return
  endif
  if (user) then
    fint2 = fint2 * usurlo(ru * u, wn)
  else
    fint2 = fint2 * surlos(ru * u, eps, thick, layers, nper)
    if (dlimf > 0.0d0) then
      t = ru * u * dlimf
      fint2 = fint2 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
    endif
  endif
  return
end function fint2

double precision function fint3(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from phi1 > 0 to phi2 < pi *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: u
  integer, intent(in) :: layers, nper, eps_size
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)

  logical :: rational, user
  double precision :: a, arg, b, ccoef, cospsi, dlimf, elleps, phi1, phi2
  double precision :: phint, sinpsi, rac, ru, um, t, tanpsi, wn

  interface
    double precision function usurlo(dq, wn)
      double precision, intent(in) :: dq
      double precision, intent(in) :: wn
    end function usurlo
    double precision function surlos(dk, eps, thick, layers, nper)
      double precision, intent(in) :: dk
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end function surlos
  end interface  
  
  common / param / a, b, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

!  write (*,*) 'fint3:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  if (u == 0.0d0) then
    fint3 = 0.0d0
    return
  endif
  rac = dsign(1.0d0, a) * cospsi * dsqrt((1.0d0 - elleps) * a * (um - u) * (um + u))
  arg = (b - rac) / (u * a)
  if (dabs(arg) > 1.0d0) arg = dsign(1.0d0, arg)
  phi2 = dacos(arg)
  fint3 = phint(phi2, tanpsi, u)
  arg = (b + rac) / (u * a)
  if (dabs(arg) > 1.0d0) arg = dsign(1.0d0, arg)
  phi1 = dacos(arg)
  fint3 = fint3 - phint(phi1, tanpsi, u)
  if (rational) return
  if (user) then
    fint3 = fint3 * usurlo(ru * u, wn)
  else
    fint3 = fint3 * surlos(ru * u, eps, thick, layers, nper)
    if (dlimf > 0.0d0) then
      t = ru * u * dlimf
      fint3 = fint3 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
    endif
  endif
  return
end function fint3

double precision function fun(phi)

! ******************************************************************
! *                                                                *
! * integrand of the expression of the 1st order term in the       *
! * expansion of the eels integral for a homogeneous target.       *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: phi

  logical :: user, rational
  double precision :: acoef, bcoef, ccoef, cospsi, dlimf, elleps, ru
  double precision :: sinphi, sinpsi, tanpsi, um, wn

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
                   ru, um, dlimf, wn, user, rational

  sinphi = dsin(phi)
  fun = dsqrt((1.0d0 - elleps + elleps * sinphi**2) *   &
              (1.0d0 - sinpsi * sinphi) *               &
              (1.0d0 + sinpsi * sinphi))
  return
end function fun

subroutine quanc8(fun, a, b, abserr, relerr, result, errest, nofun, flag, eps, thick, layers, nper)

! estimate the integral of fun(x) from a to b
! to a user provided tolerance.
! an automatic adaptive routine based on
! the 8-panel newton-cotes rule (g. forsythe et al, 1977, p. 92)
!
! input ..
!
! fun     the name of the integrand function subprogram fun(x).
! a       the lower limit of integration.
! b       the upper limit of integration.(b may be less than a.)
! relerr  a relative error tolerance. (should be non-negative)
! abserr  an absolute error tolerance. (should be non-negative)
!
! output ..
!
! result  an approximation to the integral hopefully satisfying the
!         least stringent of the two error tolerances.
! errest  an estimate of the magnitude of the actual error.
! nofun   the number of function values used in calculation of result.
! flag    a reliability indicator.  if flag is zero, then result
!         probably satisfies the error tolerance.  if flag is
!         xxx.yyy , then  xxx = the number of intervals which have
!         not converged and 0.yyy = the fraction of the interval
!         left to do when the limit on  nofun  was approached.

  implicit none

  double precision :: fun
  double precision, intent(in) :: a
  double precision, intent(in) :: b
  double precision, intent(in out) :: abserr
  double precision, intent(in) :: relerr
  double precision, intent(out) :: result
  double precision, intent(out) :: errest
  integer, intent(out) :: nofun
  double precision, intent(out) :: flag

  external fun

  double precision, intent(in) :: thick(:)
  double complex, intent(in) :: eps(:)
  integer, intent(in) :: layers, nper

  double precision :: w0, w1, w2, w3, w4, area, x0, f0, stone, step, cor11, temp
  double precision :: qprev, qnow, qdiff, qleft, esterr, tolerr
  double precision :: qright(31), f(16), x(16), fsave(8, 30), xsave(8, 30)
  double precision :: dabs, dmax1

  integer :: levmin, levmax, levout, nomax, nofin, lev, nim, i, j

! ***   stage 1 ***   general initialization
! set constants.

!  write (*,*) 'quanc8:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  levmin = 1
  levmax = 30
  levout = 6
  nomax = 5000
  nofin = nomax - 8 * (levmax - levout + 2**(levout + 1))

! trouble when nofun reaches nofin

  w0 =   3956.0d0 / 14175.0d0
  w1 =  23552.0d0 / 14175.0d0
  w2 =  -3712.0d0 / 14175.0d0
  w3 =  41984.0d0 / 14175.0d0
  w4 = -18160.0d0 / 14175.0d0

! initialize running sums to zero.

  flag   = 0.0d0
  result = 0.0d0
  cor11  = 0.0d0
  errest = 0.0d0
  area   = 0.0d0
  nofun = 0
  if (a == b) return

! ***   stage 2 ***   initialization for first interval

  lev = 0
  nim = 1
  x0 = a
  x(16) = b
  qprev  = 0.0d0
  f0 = fun(x0, eps, thick, layers, nper, size(eps))
  stone = (b - a) / 16
  x(8)  =  (x0    + x(16)) / 2
  x(4)  =  (x0    + x(8))  / 2
  x(12) =  (x(8)  + x(16)) / 2
  x(2)  =  (x0    + x(4))  / 2
  x(6)  =  (x(4)  + x(8))  / 2
  x(10) =  (x(8)  + x(12)) / 2
  x(14) =  (x(12) + x(16)) / 2
  do j = 2, 16, 2
    f(j) = fun(x(j), eps, thick, layers, nper, size(eps))
  enddo
  nofun = 9

  do

! ***   stage 3 ***   central calculation
! requires qprev, x0, x2, x4, ..., x16, f0, f2, f4, ..., f16.
! calculates x1, x3, ...x15, f1, f3, ...f15, qleft, qright, qnow, qdiff, area.

    x(1) = (x0 + x(2)) / 2
    f(1) = fun(x(1), eps, thick, layers, nper, size(eps))
    do j = 3, 15, 2
      x(j) = (x(j - 1) + x(j + 1)) / 2
      f(j) = fun(x(j), eps, thick, layers, nper, size(eps))
    enddo
    nofun = nofun + 8
    step = (x(16) - x0) / 16.0d0
    qleft  =  (w0 * (f0 + f(8))  + w1 * (f(1) + f(7)) + w2 * (f(2) + f(6))  &
        + w3 * (f(3) + f(5))  +  w4 * f(4)) * step
    qright(lev + 1) = (w0 * (f(8) + f(16)) + w1 * (f(9) + f(15)) + w2 * (f(10) + f(14))  &
        + w3 * (f(11) + f(13)) + w4 * f(12)) * step
    qnow = qleft + qright(lev + 1)
    qdiff = qnow - qprev
    area = area + qdiff

! ***   stage 4 *** interval convergence test

    esterr = dabs(qdiff) / 1023
    tolerr = dmax1(abserr, relerr * dabs(area)) * (step / stone)

    if (lev >= levmin) then
      if (lev >= levmax) then
! current level is levmax.
        flag = flag + 1.0d0
      else
        if (nofun > nofin) then
! ***   stage 6   ***   trouble section
! number of function values is about to exceed limit.
          nofin = 2 * nofin
          levmax = levout
          flag = flag + (b - x0) / (b - a)
        else
          if (esterr > tolerr) then
! ***   stage 5   ***   no convergence
! locate next interval.
            nim = 2 * nim
            lev = lev + 1
! store right hand elements for future use.
            do i = 1, 8
              fsave(i, lev) = f(i + 8)
              xsave(i, lev) = x(i + 8)
            enddo
! assemble left hand elements for immediate use.
            qprev = qleft
            do i = 1, 8
              f(18 - 2 * i) = f(9 - i)
              x(18 - 2 * i) = x(9 - i)
            enddo
            cycle
          endif
        endif
      endif

! ***   stage 7   ***   interval converged
! add contributions into running sums.
      result = result + qnow
      errest = errest + esterr
      cor11  = cor11  + qdiff / 1023
! locate next interval.
      do while (nim /= 2 * (nim / 2))
        nim = nim / 2
        lev = lev - 1
      enddo
      nim = nim + 1

      if (lev <= 0) exit

! assemble elements required for the next interval.
      qprev = qright(lev)
      x0 = x(16)
      f0 = f(16)
      do i = 1, 8
        f(2*i) = fsave(i, lev)
        x(2*i) = xsave(i, lev)
      enddo
      cycle
    else
! ***   stage 5   ***   no convergence
! locate next interval.
      nim = 2 * nim
      lev = lev + 1
! store right hand elements for future use.
      do i = 1, 8
        fsave(i, lev) = f(i + 8)
        xsave(i, lev) = x(i + 8)
      enddo
! assemble left hand elements for immediate use.
      qprev = qleft
      do i = 1, 8
        f(18 - 2 * i) = f(9 - i)
        x(18 - 2 * i) = x(9 - i)
      enddo
    endif

   enddo

 ! ***   stage 8   ***   finalize and return
  result = result + cor11

! make sure errest not less than roundoff level.
  if (errest /= 0.0d0) then
    temp = dabs(result) + errest
    do while (temp == dabs(result))
      errest = 2 * errest
      temp = dabs(result) + errest
    enddo
  endif
  return
end subroutine quanc8

subroutine queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)

! ******************************************************************
! *                                                                *
! * perform q-space integration for computing the eels spectrum of *
! * a isotropic target using polar coordinates.                    *
! *                                                                *
! * x is the dimensionless energy loss hbar*omega/(2*e0*phia)      *
! * aerr and rerr are the desired absolute and relative accuracies *
! * facru*x is the units of wavevectors omega/v_perpendicular      *
! * f is the q-integral multiplied by (2/pi)**2                    *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: x
  double precision, intent(out) :: f
  double precision, intent(in out) :: aerr
  double precision, intent(in out) :: rerr
  double precision, intent(in) :: facru
  double complex, intent(in) :: eps(:)
  double precision, intent(in) :: thick(:)
  integer, intent(in) :: layers, nper

  logical :: rational, user
  double precision :: acoef, bcoef, ccoef, cospsi, dlimf, elleps
  double precision :: error, flag, ru, sinpsi
  double precision :: u1, u2, um, ut, tanpsi, wn, y
  integer :: ie, nofu
  dimension error(3), flag(3)

  interface
    double precision function fint1(u, eps, thick, layers, nper, eps_size)
      double precision, intent(in) :: u
      integer, intent(in) :: layers, nper, eps_size
      double complex, intent(in) :: eps(eps_size)
      double precision, intent(in) :: thick(eps_size)
    end function fint1
    double precision function fint2(u, eps, thick, layers, nper, eps_size)
      double precision, intent(in) :: u
      integer, intent(in) :: layers, nper, eps_size
      double complex, intent(in) :: eps(eps_size)
      double precision, intent(in) :: thick(eps_size)
    end function fint2
    double precision function fint3(u, eps, thick, layers, nper, eps_size)
      double precision, intent(in) :: u
      integer, intent(in) :: layers, nper, eps_size
      double complex, intent(in) :: eps(eps_size)
      double precision, intent(in) :: thick(eps_size)
    end function fint3
    subroutine quanc8(fun, a, b, abserr, relerr, result, errest, nofun, flag, eps, thick, layers, nper)
      double precision :: fun
      double precision, intent(in) :: a
      double precision, intent(in) :: b
      double precision, intent(in out) :: abserr
      double precision, intent(in) :: relerr
      double precision, intent(out) :: result
      double precision, intent(out) :: errest
      integer, intent(out) :: nofun
      double precision, intent(out) :: flag

      external fun

      double precision, intent(in) :: thick(:)
      double complex, intent(in) :: eps(:)
      integer, intent(in) :: layers, nper
    end subroutine quanc8  
  end interface  

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

!  write (*,*) 'queels:'
!  write (*,*) 'eps: ', size(eps)
!  write (*,*) 'thick: ', size(thick)

  f = 0.0d0
  if (x <= 0.0d0) then
    return
  endif
  ru = facru * x
  ccoef = cospsi**2 / x
  ut = ccoef - bcoef
  u1 = dabs(ut)
  u2 = ccoef + bcoef
  if (ut > 0.0d0) then
    call quanc8(fint1, 0.0d0, u1, aerr, rerr, y, error(1), nofu, flag(1), eps, thick, layers, nper)
    f = y
  else
    flag(1) = 0.0d0
  endif
  if (u2 > u1) then
    call quanc8(fint2, u1, u2, aerr, rerr, y, error(2), nofu, flag(2), eps, thick, layers, nper)
    f = f + y
  else
    flag(2) = 0.0d0
  endif
  if (dabs(acoef) > x * (1.0d0 - elleps) * bcoef) then
    um = dsqrt(ccoef / x / (1.0d0 - elleps) + bcoef**2 / acoef)
    if (um > u2) then
      call quanc8(fint3, u2, um, aerr, rerr, y, error(3), nofu, flag(3), eps, thick, layers, nper)
      f = f + y
    endif
    if (um < u1) then
      call quanc8(fint3, um, u1, aerr, rerr, y, error(3), nofu, flag(3), eps, thick, layers, nper)
      f = f - y
    endif
  else
    flag(3) = 0.0d0
  endif
  do ie = 1, 3
    if (flag(ie) == 0.0d0) cycle
    write(*,*) ' +++ flag(', ie, ') =', flag(ie), ', error =', error(ie), ' +++'
    if (flag(ie) - aint(flag(ie)) > 0.5d-02) then
      stop '*** execution aborted ***'
    endif
  enddo
  f = (2 / 3.141592653589793238d0)**2 * f
  return
end subroutine queels

subroutine seteps(neps, nos, osc, epsinf, wn, name, eps, layers, mode)

! ******************************************************************
! *                                                                *
! * set up long-wavelength dielectric functions of the layers for  *
! * the present frequency wn (in cm**-1)                           *
! *                                                                *
! ******************************************************************

  implicit none
  integer, intent(in) :: neps
  integer, intent(in) :: nos(:)
  double precision, intent(in) :: osc(:, :)
  double precision, intent(in) :: epsinf(:)
  double precision, intent(in) :: wn
  character (len=10), intent(in) :: name(:)
  character (len=10), intent(in) :: mode

  double complex, intent(in out) :: eps(:)
  integer, intent(in) :: layers

  double precision :: argmin, argmax, epsmac, x
  double complex :: deno, nomi
  integer :: j, k, l, m

  common / mulayr / argmin, argmax, epsmac

!  write (*,*) 'seteps:'
!  write (*,*) 'nos: ', size(nos)
!  write (*,*) 'osc: ', size(osc)
!  write (*,*) 'epsinf: ', size(epsinf)
!  write (*,*) 'name: ', size(name)
!  write (*,*) 'eps: ', size(eps)
!  write (*,*) 'thick: ', size(thick)

  j = 0
  do l = 1, neps
    m = nos(l)/2
    nomi = dcmplx(1.0d0, 0.0d0)
    deno = dcmplx(1.0d0, 0.0d0)
    if (mode == 'kurosawa') then
      do k = 1, m
        j = j + 1
! since osc(1,*) and wn are real, the following should be equivalent
! Check required.
! Furthermore the first term can be rewritten as
! (osc(1, j + m) - wn) * (osc(1, j + m) + wn)
!        nomi = nomi * cmplx(osc(1, j + m)**2 - wn**2, - wn * osc(3, j + m))
        nomi = nomi * (osc(1, j + m)**2 - wn**2 - dcmplx(0.0d0, wn * osc(3, j + m)))
        deno = deno * (osc(1, j    )**2 - wn**2 - dcmplx(0.0d0, wn * osc(3, j    )))
      enddo
      eps(l) = epsinf(l) * nomi / deno
    elseif (name(l) == 'metal') then
      j = j + 1
! suggestion for replacement
!      eps(l) = -osc(1, j)**2 / cmplx(wn**2, wn * osc(3, j))
      eps(l) = -osc(1, j)**2 / ( wn**2 + dcmplx(0.0d0, wn * osc(3, j)) )
    else
      eps(l) = epsinf(l)
! The original version had this additional loop. It seems, it has been removed
! because all cases of nos(l) > 1 are treated in case 1 above
      do k = 1, nos(l)
      j = j + 1
      x = wn / osc(1, j)
      deno = x * dcmplx(x, osc(3, j))
      if (osc(2, j) >= 0.0d0) then
        deno = 1.0d0 - deno
      endif
      if (cdabs(deno) == 0.0d0) then
        deno = epsmac
      endif
      eps(l) = eps(l) + osc(2, j) / deno
      enddo
    endif
  enddo
  if (neps == layers + 1) then
! the substrate is a anisotropic uniaxial material
    eps(layers) = cdsqrt(eps(layers) * eps(layers + 1))
    if (dimag(eps(layers)) < 0.0d0) then
      eps(layers) = -eps(layers)
    endif
  endif
  return
end subroutine seteps

subroutine doeels (e0, theta, phia, phib, wmin, wmax, dw, comment, comment_size,   &
              layers, neps, nper, name, name_size, thick, epsinf, nos, osc, osc_size,&
              contrl, mode, wn_array, f_array, wn_array_size)

! ******************************************************************
! *                                                                *
! * compute the classical eels spectrum of an arbitrary plane-     *
! * statified medium made from isotropic materials in specular     *
! * geometry using the dielectric theory of eels.                  *
! *                                                                *
! ******************************************************************

  implicit none

  integer, parameter :: nt = 5

  double precision, intent(in) :: e0, theta, phia, phib, wmin, wmax, dw
  character (len = 72) :: comment(comment_size)
  character (len = 10) :: name(name_size)
  double precision, intent(in) :: thick(name_size), epsinf(name_size), osc(3, osc_size)
  character (len = 10) :: contrl, mode
  integer, intent(in) :: comment_size, name_size, osc_size, wn_array_size
  integer, intent(in out) :: layers, nper, nos(name_size)
  double precision, intent(in out) :: wn_array(wn_array_size), f_array(wn_array_size)

  logical :: rational, user, debug
  integer :: i, iw, neps, nofu, nout, nw, lmax
  double precision :: a, acoef, aerr, alpha, argmin, argmax, b, bcoef, beta,  &
      c1, c2, ccoef, cospsi, dlimf, dx, elleps, ener, epsmac, facru, f, f0,   &
      f1, fpic, fun, pi, prefac, psia, psii, qrat, rerr, ru, sinpsi, t,       &
      tanpsi, table, um, widt, wn, wpic, x, xmin, xmax, z, z1, z2
  double complex, allocatable :: eps(:)
  dimension table(nt)

  external fun, qrat

  interface
    subroutine queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)
      double precision, intent(in) :: x
      double precision, intent(out) :: f
      double precision, intent(in out) :: aerr
      double precision, intent(in out) :: rerr
      double precision, intent(in) :: facru
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end subroutine queels
    subroutine seteps(neps, nos, osc, epsinf, wn, name, eps, layers, mode)
      integer, intent(in) :: neps
      integer, intent(in) :: nos(:)
      double precision, intent(in) :: osc(:, :)
      double precision, intent(in) :: epsinf(:)
      double precision, intent(in) :: wn
      character (len=10), intent(in) :: name(:)
      character (len=10), intent(in) :: mode
      double complex, intent(in out) :: eps(:)
      integer, intent(in) :: layers
    end subroutine seteps
    subroutine quanc8(fun, a, b, abserr, relerr, result, errest, nofun, flag, eps, thick, layers, nper)
      double precision :: fun
      double precision, intent(in) :: a
      double precision, intent(in) :: b
      double precision, intent(in out) :: abserr
      double precision, intent(in) :: relerr
      double precision, intent(out) :: result
      double precision, intent(out) :: errest
      integer, intent(out) :: nofun
      double precision, intent(out) :: flag

      external fun

      double precision, intent(in) :: thick(:)
      double complex, intent(in) :: eps(:)
      integer, intent(in) :: layers, nper
    end subroutine quanc8  
  end interface

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
                   ru, um, dlimf, wn, user, rational
  common / mulayr / argmin, argmax, epsmac

  data aerr / 0.0d0 /, rerr / 1.0d-06 /, f / 0.0d0 /, f1 / 0.0d0 /

  debug = .false.
  if (debug) then
    write (*,*) 'doeels:'
    write (*,*) 'comment: ',  size(comment)
    write (*,*) 'name: ',     size(name)
    write (*,*) 'thick: ',    size(thick)
    write (*,*) 'epsinf: ',   size(epsinf)
    write (*,*) 'osc: ',      size(osc), size(osc, 1), size(osc, 2)
    write (*,*) 'nos: ',      size(nos)
    write (*,*) 'wn_array: ', size(wn_array)
    write (*,*) 'f_array: ',  size(f_array)
  endif

! *** machine-dependent constants
! *** epsmac + 1.0 = epsmac , cosh(argmin) = 1.0 , tanh(argmax) = 1.0

  pi = 4 * datan(1.0d0)
  epsmac = 1.0d0
  do while (1.0d0 + epsmac > 1.0d0)
    epsmac = epsmac / 2
  enddo
  argmin = dsqrt(2 * epsmac)
  argmax = 0.5d0 * dlog(2 / epsmac)

  dlimf = 0.0d0
  rational = .false.

! *** read target specifications

  user = layers == 0
  if (user) then

    if (layers == 1) rational = .true.
    if (contrl == 'image') then
! *** image-charge screening factor
      if (layers == 1 .and. neps == 2) then
        dlimf = dsqrt(epsinf(1) * epsinf(2))
      else
        dlimf = epsinf(1)
      endif
      dlimf = (dlimf - 1.0d0) / (dlimf + 1.0d0)
    endif
  endif

! *** initialize constants

  lmax = size(thick)
  nw = size(wn_array)
  if (debug) write (*,*) 'lmax: ', lmax
  allocate(eps(lmax))
  if (debug) write (*,*) 'eps: ', size(eps)
  nout = 1 + nw / 20
  ener = 8065 * e0
  psia = phia / 180 * pi
  psii = theta / 180 * pi
  cospsi = dcos(psii)
  sinpsi = dsin(psii)
  tanpsi = dtan(psii)
  prefac = dsqrt(255500 / e0)/(137 * cospsi)
  facru = psia / cospsi * dsqrt(0.2624664d0 * e0)
  elleps = (1.0d0 - phia / phib) * (1.0d0 + phia / phib)
  acoef = sinpsi**2 + elleps * cospsi**2
  bcoef = sinpsi * cospsi
  if (dlimf > 0.0d0) then
    rational = .false.
    if (debug) write(*,*) ' = > electron attracted by an image charge = ', dlimf
! *** dlimf : half the length unit imposed by the image force
    dlimf = 1.80d0 * dlimf/(e0 * cospsi**2)
  endif
  if (debug) write (*,*) 'rational: ', rational
  if (rational) then

! *** set up coefficients for the rational approximation to the integral

    if (debug) write(*,*) ' = > set up a rational approximation to the integral'
    call quanc8(fun, 0.0d0, pi / 2, aerr, rerr, alpha, c1, nofu, c2, eps, thick, layers, nper)
    alpha  = (2 / pi)**2 * alpha
    c1 = 2 / pi / dsqrt(1.0d0 - elleps) * sinpsi * alpha**2
    if (c1 > 0.99d0) then
      if (debug) write(*,*) ' ===> cannot do it'
      rational = .false.
    else
      c2 = 3 * alpha**2 / (1.0d0 - c1)
      c1 = c1 * c2
      xmin = wmin / (2 * ener * psia)
      xmax = wmax / (2 * ener * psia)
      if (xmin <= 0.0d0) xmin = 0.0d0
      dx = dmax1(0.02d0, (xmax - xmin) / nt)
      z1 = 0.0d0
      z2 = 0.0d0
      do i = 1, nt
        x = xmin + i * dx
        call queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)
        table(i) = f
        f = f * (1.0d0 + alpha * x)**2
        if (dabs(c2 * f - c1) < c2 * rerr) cycle
        z = (1.0d0 - f) / (c2 * f - c1)
        if (z <= 0.0d0) cycle
        z1 = z1 + x * z * (x**2 - z)
        z2 = z2 + (x * z)**2
      enddo
      if (z2 == 0.0d0) then
        if (debug) write(*,*) ' ===> cannot do it'
        rational = .false.
      else
        beta = z1 / z2
        z = 0.0d0
        do i = 1, nt
          x = xmin + i * dx
          z = z + (table(i) - qrat(x, alpha, beta, c1, c2))**2
        enddo
        z = dsqrt(z) / nt
        if (z > 5.0d-03) then
          if (debug) write(*,*) ' ===> cannot do it'
          rational = .false.
        else
          if (debug) write(*, 100) alpha, c1, c2, beta, z
        endif ! z > 5.0d-03
      endif ! z2 == 0.0d0
    endif ! c1 > 0.99d0
  endif ! rational

! *** loop over the energy losses

  if (debug) write(*, 110)
  do iw = 1, nw
    f0 = f1
    f1 = f
    f = 0.0d0
    wn = wmin + (iw - 1) * dw
!    if (debug) write (*,*) 'wn: ', wn
    if (wn >= 0.0d0) then
      if (wn /= 0.0d0) then
        if (.not. user) call seteps(neps, nos, osc, epsinf, wn, name, eps, layers, mode)

        x = wn / (2 * ener * psia)
        if (rational) then
          f = qrat(x, alpha, beta, c1, c2) * dimag(-2 / (1.0d0 + eps(1)))
        else
          call queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)
        endif
        f = prefac * f / wn
      endif ! wn /= 0.0d0

      wn_array(iw) = wn
      f_array(iw)  = f

! *** localize a peak using a parabolic interpolation

      if ((iw >= 3) .and. (f1 - f0 > aerr) .and. (f1 - f > aerr)) then
        a = (f1 - f0) + (f1 - f)
        if (a > 4 * rerr * f1) then
          b = 0.5d0 * (f1 - f0 + 3 * (f1 - f))
          t = b / a
          wpic = wn - t * dw
          fpic = f + 0.5d0 * b * t
          widt = dsqrt(8 * fpic / a) * dw
          if (debug) write(*, 120) wpic, fpic, widt
        endif ! a > 4 * rerr * f1
      endif ! iw >= 3 ...
    endif ! wn >= 0.0d0
    if (mod(iw, nout) == 0) then
      if (debug) write(*, 130) 100 * iw / nw, wn, f
    endif
  enddo
  return
100 format(5x, 'alpha = ', f9.4, 4x, 'c1 = ', f9.4, 4x, 'c2 = ', f9.4, 4x,  &
      'beta = ', f9.4/5x, 'accuracy = ', e9.2)
110 format(//' run (%)  wn (cm**-1)  pcl(wn) (cm) |',  &
      ' peak location  amplitude    width')
120 format(40x, f10.2, d12.4, f10.2)
130 format(2x, f5.1, 3x, f11.3, d14.5)
end subroutine doeels

subroutine change_working_dir()

! This routine gets the first argument of the commandline and takes it
! as the path to change the working directory
! used intrinsic routines:
! iarg returns the number of commandline arguments without the program cname.
! chdir changes the directory and returns 0 on success.
! trim removes blanks from strings.

  character (len = 256) :: argument
  integer :: status

  if (iargc() == 1) then
    call getarg(1, argument)
    status = chdir(trim(argument))
    if (status /= 0) then
      write (*,*) '*** change directory failed ***'
      write (*,*) 'directory tried: ', trim(argument)
      write (*,*) 'error code (see: man chdir): ', status
      write (*,*) 'continuing in the start directory'
    end if
  end if

  return
end subroutine change_working_dir
