module fint1_mod
contains
double precision function fint1(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from 0.0 to pi             *
! *                                                                *
! ******************************************************************

  use surlos_mod

  implicit none

  double precision, intent(in) :: u
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)
  integer, intent(in) :: layers, nper, eps_size

  logical :: rational, user
  double precision :: acoef, bcoef, ccoef, cospsi, den, dif, dlimf, e, elleps
  double precision :: pi, rom, rop, ru, sinpsi, sum, um, usurlo, t
  double precision :: tanpsi, wn, u2

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

  data pi / 3.141592653589793238d0 /

!  write (*,*) 'fint1:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  if (u == 0.0d0) then
    fint1 = 0.0d0
    return
  endif
  e = tanpsi * u
  u2 = u**2
  rom = (1.0d0 - e)**2 + u2
  rop = (1.0d0 + e)**2 + u2
  sum = rop + rom
  rom = dsqrt(rom)
  rop = dsqrt(rop)
  dif = rop - rom
  den = dsqrt((2.0d0 - dif) * (2.0d0 + dif)) * rop * rom
  fint1 = pi * u2 * (4 * sum - dif**2 * (sum - rop * rom)) / den**3
  if (rational) then
    return
  endif
  if (user) then
    fint1 = fint1 * usurlo(ru * u, wn)
  else
    fint1 = fint1 * surlos(ru * u, eps, thick, layers, nper)
    if (dlimf > 0.0d0) then
      t = ru * u * dlimf
      fint1 = fint1 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
    endif
  endif
  return
end function fint1
end module fint1_mod
