module fint3_mod
contains
double precision function fint3(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from phi1 > 0 to phi2 < pi *
! *                                                                *
! ******************************************************************

  use surlos_mod

  implicit none

  double precision, intent(in) :: u
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)
  integer, intent(in) :: layers, nper, eps_size

  logical :: rational, user
  double precision :: a, arg, b, ccoef, cospsi, dlimf, elleps, phi1, phi2
  double precision :: phint, sinpsi, rac, ru, um, usurlo, t, tanpsi, wn

  common / param / a, b, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

!  write (*,*) 'fint3:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  if (u == 0.0d0) then
    fint3 = 0.0d0
    return
  endif
  rac = dsign(1.0d0, a) * cospsi * dsqrt((1.0d0 - elleps) * a * (um - u) * (um + u))
  arg = (b - rac) / (u * a)
  if (dabs(arg) > 1.0d0) arg = dsign(1.0d0, arg)
  phi2 = dacos(arg)
  fint3 = phint(phi2, tanpsi, u)
  arg = (b + rac) / (u * a)
  if (dabs(arg) > 1.0d0) arg = dsign(1.0d0, arg)
  phi1 = dacos(arg)
  fint3 = fint3 - phint(phi1, tanpsi, u)
  if (rational) return
  if (user) then
    fint3 = fint3 * usurlo(ru * u, wn)
  else
    fint3 = fint3 * surlos(ru * u, eps, thick, layers, nper)
    if (dlimf > 0.0d0) then
      t = ru * u * dlimf
      fint3 = fint3 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
    endif
  endif
  return
end function fint3
end module fint3_mod
